import pathlib
import collections
import json

import yaml
from cookiecutter.main import cookiecutter
from cookiecutter.generate import generate_files


def patch_dask_gateway_extra_config(config):
    """Basically the current dask_gateway helm chart only allows one
    update to extraContainerConfig and extraPodConfig for the workers
    and scheduler. Thus we need to copy the configuration done in
    these settings. The only critical one is mounting the conda store
    directory.

    """
    conda_store_volume = {
        "name": "conda-store",
        "persistentVolumeClaim": {"claimName": "conda-store-dev-share"},
    }

    if "profiles" in config and "dask_worker" in config["profiles"]:
        for worker_name, worker_config in config["profiles"]["dask_worker"].items():
            if "worker_extra_pod_config" in worker_config:
                if "volumes" in worker_config["worker_extra_pod_config"]:
                    worker_config["worker_extra_pod_config"]["volumes"].append(
                        conda_store_volume
                    )
                else:
                    worker_config["worker_extra_pod_config"]["volumes"] = [
                        conda_store_volume
                    ]


def render_default_template(output_directory, config_filename=None, force=False):
    import qhub_ops

    input_directory = pathlib.Path(qhub_ops.__file__).parent / "template"
    render_template(input_directory, output_directory, config_filename, force=force)


def render_template(
    input_directory, output_directory, config_filename=None, force=False
):
    # would be nice to remove assumption that input directory
    # is in local filesystem
    input_directory = pathlib.Path(input_directory)
    if not input_directory.is_dir():
        raise ValueError(f"input directory={input_directory} is not a directory")

    output_directory = pathlib.Path(output_directory).resolve()
    # due to cookiecutter requiring a template directory folder
    # we take the output directory and split into two components
    repo_directory = output_directory.name
    output_directory = output_directory.parent
    output_directory.mkdir(exist_ok=True, parents=True)

    prompt_filename = input_directory / "hooks" / "prompt_gen_project.py"

    if config_filename is not None:
        filename = pathlib.Path(config_filename)

        if not filename.is_file():
            raise ValueError(f"cookiecutter configuration={filename} is not filename")

        with filename.open() as f:
            config = yaml.safe_load(f)
            config["repo_directory"] = repo_directory
            patch_dask_gateway_extra_config(config)

        with (input_directory / "cookiecutter.json").open() as f:
            config = collections.ChainMap(config, json.load(f))

        generate_files(
            repo_dir=str(input_directory),
            context={"cookiecutter": config},
            output_dir=str(output_directory),
            overwrite_if_exists=force,
        )
    elif prompt_filename.is_file():
        with prompt_filename.open() as f:
            content = f.read()

        global_context = {}
        exec(content, global_context, global_context)
        config = global_context["COOKIECUTTER_CONFIG"]

        cookiecutter(
            str(input_directory),
            no_input=True,
            extra_context=config,
            output_dir=str(output_directory),
            overwrite_if_exists=force,
        )
    else:
        cookiecutter(
            str(input_directory),
            output_dir=str(output_directory),
            overwrite_if_exists=force,
        )



import yaml
import pathlib
import logging

from qhub_ops.deploy import deploy_configuration

logger = logging.getLogger(__name__)


def create_deploy_subcommand(subparser):
    subparser = subparser.add_parser("deploy")
    subparser.add_argument("-c", "--config", help="qhub configuration", required=True)
    subparser.set_defaults(func=handle_deploy)


def handle_deploy(args):
    config_filename = pathlib.Path(args.config)
    if not config_filename.is_file():
        raise ValueError(
            f"passed in configuration filename={config_filename} must exist"
        )

    with config_filename.open() as f:
        config = yaml.safe_load(f.read())

    deploy_configuration(config)



import os
import logging

import CloudFlare


logger = logging.getLogger(__name__)


def update_record(zone_name, record_name, record_type, record_address):
    CLOUDFLARE_API_TOKEN = os.environ["CLOUDFLARE_TOKEN"]
    cf = CloudFlare.CloudFlare(token=CLOUDFLARE_API_TOKEN)

    record = {
        "name": record_name,
        "type": record_type,
        "content": record_address,
        "ttl": 1,
        "proxied": False,
    }

    zone_id = None
    for zone in cf.zones.get():
        if zone["name"] == zone_name:
            zone_id = zone["id"]
            break
    else:
        raise ValueError(f"Cloudflare zone {zone_name} not found")

    for _record in cf.zones.dns_records.get(zone_id):
        if _record["name"] == record_name:
            logger.info(
                f"record name={record_name} type={record_type} address={record_address} already exists updating"
            )
            cf.zones.dns_records.put(zone_id, _record["id"], data=_record)
            break
    else:
        logger.info(
            f"record name={record_name} type={record_type} address={record_address} does not exists creating"
        )
        cf.zones.dns_records.post(zone_id, data=record)



import subprocess
import json
import functools


@functools.lru_cache()
def projects():
    output = subprocess.check_output(["gcloud", "projects", "list", "--format=json"])
    data = json.loads(output.decode("utf-8"))
    return {_["name"]: _["projectId"] for _ in data}


@functools.lru_cache()
def regions(project):
    output = subprocess.check_output(
        ["gcloud", "compute", "regions", "list", "--project", project, "--format=json"]
    )
    data = json.loads(output.decode("utf-8"))
    return {_["description"]: _["name"] for _ in data}


@functools.lru_cache()
def zones(project, region):
    output = subprocess.check_output(
        ["gcloud", "compute", "zones", "list", "--project", project, "--format=json"]
    )
    data = json.loads(output.decode("utf-8"))
    return {_["description"]: _["name"] for _ in data if _["name"].startswith(region)}


@functools.lru_cache()
def kubernetes_versions(region):
    output = subprocess.check_output(
        [
            "gcloud",
            "container",
            "get-server-config",
            "--region",
            region,
            "--format=json",
        ]
    )
    data = json.loads(output.decode("utf-8"))
    return {_: _ for _ in data["validMasterVersions"]}


@functools.lru_cache()
def instances(project):
    output = subprocess.check_output(
        [
            "gcloud",
            "compute",
            "machine-types",
            "list",
            "--project",
            project,
            "--format=json",
        ]
    )
    data = json.loads(output.decode("utf-8"))
    return {_["description"]: _["name"] for _ in data}


# Getting pricing data could come from here
# https://cloudpricingcalculator.appspot.com/static/data/pricelist.json



import subprocess
import logging
import json

from qhub_ops.utils import timer

logger = logging.getLogger(__name__)


def init(directory=None):
    logger.info(f"terraform init directory={directory}")
    with timer(logger, "terraform init"):
        subprocess.check_output("terraform init", shell=True, cwd=directory)


def apply(directory=None, targets=None):
    targets = targets or []

    logger.info(f"terraform apply directory={directory} targets={targets}")
    with timer(logger, "terraform apply"):
        command = " ".join(
            ["terraform", "apply", "-auto-approve"] + ["-target=" + _ for _ in targets]
        )

        subprocess.check_output(command, shell=True, cwd=directory)


def output(directory=None):
    logger.info(f"terraform output directory={directory}")
    with timer(logger, "terraform output"):
        output = subprocess.check_output(
            "terraform output -json", shell=True, cwd=directory
        ).decode("utf8")
        return json.loads(output)


def destroy(directory=None):
    logger.info(f"terraform destroy directory={directory}")

    with timer(logger, "terraform destroy"):
        command = "terraform destroy -auto-approve"
        subprocess.check_output(command, shell=True, cwd=directory)



import re
import sys

valid_project_name = r"[A-Za-z0-9\-_.]+"

PROJECT_NAME = "{{ cookiecutter.project_name }}"
ENDPOINT = "{{ cookiecutter.endpoint }}"

if not re.fullmatch(r"[A-Za-z0-9\-_.]+", PROJECT_NAME):
    print(
        'ERROR: The project name (%s) is not a valid project name. Please use only A-Z a-z 0-9 "-" "_" "."'
        % PROJECT_NAME
    )
    sys.exit(1)


if not re.fullmatch(r"[A-Za-z0-9.]+", ENDPOINT):  # naive hostname check
    print("ERROR: The endpoint (%s) is not valid" % PROJECT_NAME)
    sys.exit(1)



import pytest

from qhub_ops.render import render_default_template


@pytest.mark.parametrize('config_filename', [
    'tests/assets/config_aws.yaml',
    'tests/assets/config_gcp.yaml',
    'tests/assets/config_do.yaml',
])
def test_render(config_filename, tmp_path):
    output_directory = tmp_path / 'test'

    render_default_template(str(output_directory), config_filename)



import argparse
import sys
import logging

from qhub_ops.cli.deploy import create_deploy_subcommand
from qhub_ops.cli.render import create_render_subcommand


def cli(args):
    parser = argparse.ArgumentParser(description="QHub command line")
    parser.set_defaults(func=None)

    subparser = parser.add_subparsers(help="QHub Ops")
    create_deploy_subcommand(subparser)
    create_render_subcommand(subparser)

    args = parser.parse_args(args)

    if args.func is None:
        parser.print_help(sys.stderr)
        sys.exit(1)

    logging.basicConfig(level=logging.INFO)

    args.func(args)



import yaml
import pathlib
import logging
import re

from qhub_ops.render import render_default_template
from qhub_ops.provider.oauth import auth0
from qhub_ops.provider.dns import cloudflare
from qhub_ops.provider import terraform
from qhub_ops.utils import timer

logger = logging.getLogger(__name__)


def deploy_configuration(config):
    logger.info(f'All qhub endpoints will be under *.{config["domain"]}')

    jupyterhub_endpoint = f'jupyter.{config["domain"]}'
    if (
        "client_id" not in config["authentication"]["config"]
        or "client_secret" not in config["authentication"]["config"]
    ):
        logger.info(
            "client_id and client_secret were not specified - dynamically creating oauth client"
        )
        with timer(logger, "creating oauth client"):
            config["authentication"]["config"] = auth0.create_client(
                jupyterhub_endpoint
            )

    with timer(logger, "rendering template"):
        tmp_config = pathlib.Path("./config.yaml")
        with tmp_config.open("w") as f:
            yaml.dump(config, f)

        render_default_template(".", tmp_config)

    infrastructure_dir = pathlib.Path(config["project_name"]) / "infrastructure"

    terraform.init(str(infrastructure_dir))

    # ========= boostrap infrastructure ========
    terraform.apply(
        str(infrastructure_dir),
        targets=[
            "module.kubernetes",
            "module.kubernetes-initialization",
            "module.kubernetes-ingress",
        ],
    )

    # ============= update dns ================
    output = terraform.output(str(infrastructure_dir))
    for key in output:
        if key.startswith("ingress"):
            endpoint = f'{key.split("_")[1]}.{config["domain"]}'
            address = output[key]["value"]
            if re.fullmatch(r"\d+\.\d+\.\d+\.\d+", address):
                cloudflare.update_record("qhub.dev", endpoint, "A", address)
            else:
                cloudflare.update_record("qhub.dev", endpoint, "CNAME", address)

    # ======= apply entire infrastructure ========
    terraform.apply(str(infrastructure_dir))



from collections import OrderedDict

import click
from cookiecutter import prompt
import yaml

from qhub_ops.provider.cloud import digital_ocean, google_cloud, amazon_web_services


DEFAULT_CONFIGURATION = {
    "project_name": "qhub-jupyterhub",
    "providers": {
        "Google Cloud Platform (GCP)": "gcp",
        "Amazon Web Services (AWS)": "aws",
        "Digital Ocean (DO)": "do",
    },
    "ci_cd": {"Github Actions": "github-actions"},
    "authentication": {"Github OAuth": "github"},
    "node_groups": {"general", "user", "worker"},
}


def read_user_multichoice_dict(var_name, options):
    choice_map = OrderedDict(
        ("{}".format(i), value) for i, value in enumerate(list(options.keys()), 1)
    )
    choices = choice_map.keys()
    default = "1"

    choice_lines = ["{} - {}".format(*c) for c in choice_map.items()]
    prompt = "\n".join(
        (
            "Select {}:".format(var_name),
            "\n".join(choice_lines),
            "Choose from {}".format(", ".join(choices)),
        )
    )

    user_choice = click.prompt(prompt, type=str, default=default, show_choices=False)
    return [options[choice_map[_.strip()]] for _ in user_choice.split(",")]


def prompt_list(var_name):
    return click.prompt(f'{var_name} ("," delimited)').split(",")


def prompt_dict(message, d, sort=True):
    options = list(d.keys())
    if sort:
        options = sorted(options)

    return d[prompt.read_user_choice(message, options)]


def prompt_range(message, min_value, max_value, default_value):
    while True:
        value = click.prompt(
            f"Select {message} [{min_value} - {max_value}]",
            type=int,
            default=default_value,
        )
        if min_value <= value <= max_value:
            break
    return value


def prompt_github_config(config):
    client_id = click.prompt("GitHub Client ID")
    client_secret = click.prompt("GitHub Client Secret")

    config["auth"]["type"] = "custom"
    config["auth"]["github"] = {
        "client_id": client_id,
        "client_secret": client_secret,
        "oauth_callback_url": f'https://{config["endpoint"]}/hub/oauth_callback',
    }


def prompt_auth0_config(config):
    client_id = click.prompt("Auth0 Client ID")
    client_secret = click.prompt("Auth0 Client Secret")
    auth0_subdomain = click.prompt("Auth0 Subdomain")

    config["auth"]["type"] = "custom"
    config["auth"]["custom"] = {
        "className": "oauthenticator.auth0.Auth0OAuthenticator",
        "client_id": client_id,
        "client_secret": client_secret,
        "oauth_callback_url": f'https://{config["endpoint"]}/hub/oauth_callback',
        "scopes": ["openid", "profile", "email"],
    }


def prompt_do_config(config):
    config["digital_ocean"] = {}
    config["digital_ocean"]["region"] = prompt_dict("Region", digital_ocean.regions())
    config["digital_ocean"]["kubernetes_version"] = prompt_dict(
        "Kubernetes Version", digital_ocean.kubernetes_versions()
    )

    config["digital_ocean"]["node_groups"] = {}
    for node_group in DEFAULT_CONFIGURATION["node_groups"]:
        instance_type = prompt_dict(
            f"{node_group} node group instance type", digital_ocean.instances()
        )
        min_nodes = prompt_range(f"{node_group} node group min nodes", 1, 999_999, 1)
        max_nodes = prompt_range(
            f"{node_group} node group min nodes", min_nodes, 999_999, min_nodes
        )

        config["digital_ocean"]["node_groups"][node_group] = {
            "instance": instance_type,
            "min_nodes": min_nodes,
            "max_nodes": max_nodes,
        }


def prompt_gcp_config(config):
    project = prompt_dict("Project", google_cloud.projects())
    config["google_cloud_platform"]["project"] = project
    region = prompt_dict("Region", google_cloud.regions(project))
    config["google_cloud_platform"]["region"] = region
    config["google_cloud_platform"]["zone"] = prompt_dict(
        "Zone", google_cloud.zones(project, region)
    )
    availability_zones = read_user_multichoice_dict(
        "Availability Zone", google_cloud.zones(project, region)
    )
    config["google_cloud_platform"]["availability_zones"] = str(
        availability_zones
    ).replace("'", '"')
    config["google_cloud_platform"]["kubernetes_version"] = prompt_dict(
        "Kubernetes Version", google_cloud.kubernetes_versions(region)
    )

    config["google_cloud_platform"]["node_groups"] = {}
    for node_group in DEFAULT_CONFIGURATION["node_groups"]:
        instance_type = prompt_dict(
            f"{node_group} node group instance type", google_cloud.instances(project),
        )
        min_nodes = prompt_range(f"{node_group} node group min nodes", 0, 999_999, 1)
        max_nodes = prompt_range(
            f"{node_group} node group min nodes", min_nodes, 999_999, min_nodes
        )

        config["google_cloud_platform"]["node_groups"][node_group] = {
            "instance": instance_type,
            "min_nodes": min_nodes,
            "max_nodes": max_nodes,
        }


def prompt_aws_config(config):
    region = prompt_dict("Region", amazon_web_services.regions())
    config["amazon_web_services"]["region"] = region
    availability_zones = read_user_multichoice_dict(
        "Availability Zones", amazon_web_services.zones(project, region)
    )
    config["amazon_web_services"]["availability_zones"] = str(
        availability_zones
    ).replace("'", '"')
    config["amazon_web_services"]["kubernetes_version"] = prompt_dict(
        "Kubernetes Version", amazon_web_services.kubernetes_versions()
    )

    config["amazon_web_services"]["node_groups"] = {}
    for node_group in DEFAULT_CONFIGURATION["node_groups"]:
        instance_type = prompt_dict(
            f"{node_group} node group instance type",
            amazon_web_services.instances(region),
        )
        min_nodes = prompt_range(f"{node_group} node group min nodes", 0, 999_999, 1)
        max_nodes = prompt_range(
            f"{node_group} node group min nodes", min_nodes, 999_999, min_nodes
        )

        config["amazon_web_services"]["node_groups"][node_group] = {
            "instance": instance_type,
            "min_nodes": min_nodes,
            "max_nodes": max_nodes,
        }


def prompt_config():
    project_name = prompt.read_user_variable(
        "Project Name", DEFAULT_CONFIGURATION["project_name"]
    )

    ci_cd = prompt_dict("Continuous Delivery", DEFAULT_CONFIGURATION["ci_cd"])

    endpoint = prompt.read_user_variable("Jupyterhub Endpoint")

    authentication = prompt_dict(
        "Authentication", DEFAULT_CONFIGURATION["authentication"]
    )

    provider = prompt_dict("Cloud Provider", DEFAULT_CONFIGURATION["providers"])

    config = {
        "project_name": project_name,
        "provider": provider,
        "ci_cd": ci_cd,
        "endpoint": endpoint,
    }

    auth_config = {"auth": {"admin": {"access": True, "users": []}, "whitelist": []}}

    if authentication == "github":
        prompt_github_auth_config(auth_config)
    elif authentication == "auth0":
        prompt_auth0_config(auth_config)

    config["authentication"] = yaml.dumps(auth_config)

    if provider == "do":
        prompt_do_config(config)
    elif provider == "gcp":
        prompt_gcp_config(config)
    elif provider == "aws":
        prompt_aws_config(config)
    return config


COOKIECUTTER_CONFIG = prompt_config()



import os
import yaml

PROJECT_DIRECTORY = os.path.realpath(os.path.curdir)
PROVIDER = "{{ cookiecutter.provider }}"
ENVIRONMENTS = eval("{{ cookiecutter.environments }}")


def remove_file(filepath):
    os.remove(os.path.join(PROJECT_DIRECTORY, filepath))


if __name__ == "__main__":
    if ENVIRONMENTS:
        os.makedirs("environments", exist_ok=True)
        for name, spec in ENVIRONMENTS.items():
            with open(f"environments/{name}", "w") as f:
                yaml.dump(spec, f)

    if PROVIDER == "aws":
        remove_file("infrastructure/do.tf")
        remove_file("infrastructure/gcp.tf")
    elif PROVIDER == "do":
        remove_file("infrastructure/aws.tf")
        remove_file("infrastructure/gcp.tf")
    elif PROVIDER == "gcp":
        remove_file("infrastructure/aws.tf")
        remove_file("infrastructure/do.tf")



import subprocess
import json
import functools


@functools.lru_cache()
def instances():
    output = subprocess.check_output(
        ["doctl", "kubernetes", "options", "sizes", "-o=json"]
    )
    data = json.loads(output.decode("utf-8"))
    return {_["name"]: _["slug"] for _ in data}


@functools.lru_cache()
def regions():
    output = subprocess.check_output(
        ["doctl", "kubernetes", "options", "regions", "-o=json"]
    )
    data = json.loads(output.decode("utf-8"))
    return {_["name"]: _["slug"] for _ in data}


@functools.lru_cache()
def kubernetes_versions():
    output = subprocess.check_output(
        ["doctl", "kubernetes", "options", "versions", "-o=json"]
    )
    data = json.loads(output.decode("utf-8"))
    return {_["kubernetes_version"]: _["slug"] for _ in data}



from qhub_ops.render import render_default_template, render_template


def create_render_subcommand(subparser):
    subparser = subparser.add_parser("render")
    subparser.add_argument("-i", "--input", help="input directory")
    subparser.add_argument(
        "-f", "--force", action="store_true", help="overwrite existing files"
    )
    subparser.add_argument("-o", "--output", default="./", help="output directory")
    subparser.add_argument("-c", "--config", help="cookiecutter configuration")
    subparser.set_defaults(func=handle_render)


def handle_render(args):
    if args.input is None:
        render_default_template(args.output, args.config, force=args.force)
    else:
        render_template(args.input, args.output, args.config, force=args.force)



import yaml
import json
from jinja2.ext import Extension


class YamlifyExtension(Extension):
    """Jinja2 extension to convert a Python object to YAML."""

    def __init__(self, environment):
        """Initialize the extension with the given environment."""
        super().__init__(environment)

        def yamlify(obj):
            return yaml.dump(obj)

        environment.filters["yamlify"] = yamlify


class JsonifyExtension(Extension):
    """Jinja2 extension to convert a Python object to JSON."""

    def __init__(self, environment):
        """Initialize the extension with the given environment."""
        super().__init__(environment)

        def jsonify(obj):
            return json.dumps(obj)

        environment.filters["jsonify"] = jsonify



from setuptools import setup, find_packages
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='qhub-ops',
    version='0.1.6',
    description='Management of QHub on Cloud Infrastructure',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/quansight/qhub-ops',
    author='Quansight',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Intended Audience :: Developers',
        'Topic :: Software Development :: Build Tools',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
    ],
    keywords='aws gcp do qhub',
    packages=find_packages(),
    python_requires='>=2.7, !=3.0.*, !=3.1.*, !=3.2.*, !=3.3.*, !=3.4.*, !=3.5.*, <4',
    install_requires=[
        "cookiecutter",
        "pyyaml",
        "cloudflare",
        "auth0-python"
    ],
    extras_require={
        'dev': [
            'flake8',
            'black',
            'twine',
            'pytest'
        ]
    },
    include_package_data=True,
    entry_points={
        'console_scripts': [
            'qhub-ops = qhub_ops.__main__:main',
        ],
    },
    project_urls={  # Optional
        'Bug Reports': 'https://github.com/quansight/QHub',
        'Source': 'https://github.com/quansight/QHub',
    },
)



import os

from auth0.v3.management import Auth0
from auth0.v3.authentication import GetToken


def create_client(jupyterhub_endpoint):
    DOMAIN = os.environ["AUTH0_DOMAIN"]
    CLIENT_ID = os.environ["AUTH0_CLIENT_ID"]
    CLIENT_SECRET = os.environ["AUTH0_CLIENT_SECRET"]

    get_token = GetToken(DOMAIN)
    token = get_token.client_credentials(
        CLIENT_ID, CLIENT_SECRET, "https://{}/api/v2/".format(DOMAIN)
    )
    mgmt_api_token = token["access_token"]

    auth0 = Auth0(DOMAIN, mgmt_api_token)

    credentials = auth0.clients.create(
        {
            "name": f"QHub - {jupyterhub_endpoint}",
            "description": f"QHub - {jupyterhub_endpoint}",
            "callbacks": [f"https://{jupyterhub_endpoint}/hub/oauth_callback"],
            "app_type": "regular_web",
        }
    )

    return {
        "auth0_subdomain": ".".join(DOMAIN.split(".")[:-2]),
        "client_id": credentials["client_id"],
        "client_secret": credentials["client_secret"],
        "scope": ["openid", "email", "profile"],
        "oauth_callback_url": f"https://{jupyterhub_endpoint}/hub/oauth_callback",
    }



import json
import subprocess
import functools


@functools.lru_cache()
def regions():
    output = subprocess.check_output(["aws", "ec2", "describe-regions"])
    data = json.loads(output.decode("utf-8"))
    return {_["RegionName"]: _["RegionName"] for _ in data["Regions"]}


@functools.lru_cache()
def zones(region):
    output = subprocess.check_output(
        ["aws", "ec2", "describe-availability-zones", "--region", region]
    )
    data = json.loads(output.decode("utf-8"))
    return {_["ZoneName"]: _["ZoneName"] for _ in data["AvailabilityZones"]}


@functools.lru_cache()
def kubernetes_versions():
    return {_: _ for _ in ["1.13", "1.14", "1.15"]}


@functools.lru_cache()
def instances(region):
    output = subprocess.check_output(
        ["aws", "ec2", "describe-instance-types", "--region", region]
    )
    data = json.loads(output.decode("utf-8"))
    return {_["InstanceType"]: _["InstanceType"] for _ in data["InstanceTypes"]}
